package e2e

import (
	"fmt"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/client/channel"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/client/ledger"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/client/resmgmt"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/common/errors/retry"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/common/providers/core"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/common/providers/fab"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/core/config"
	packager "github.com/fabric-sdk-sample/fabric-sdk-go/pkg/fab/ccpackager/gopackager"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/fabsdk"
	"github.com/fabric-sdk-sample/fabric-sdk-go/test/integration"
	"github.com/fabric-sdk-sample/fabric-sdk-go/third_party/github.com/hyperledger/fabric/common/cauthdsl"
	"github.com/fabric-sdk-sample/fabric-sdk-go/third_party/github.com/hyperledger/fabric/protos/common"
)

var (
	MychannelID          = "mychannel"
	MyorgName            = "org2"
	MyccID               = "mycc"
	MydefaultQueryArgs   = [][]byte{[]byte("b")}
	MydefaultexecuteArgs = [][]byte{[]byte("b"), []byte("a"), []byte("10")}
)

func getConfig() core.ConfigProvider {
	configPath := "../../fixtures/config/config_e2e.yaml"
	return config.FromFile(configPath)
}

/*
Parameters:
  ccId: is chaincode name(such mycc)
  functionName: is query chaincode functionName (such query)
  ccArgs: is query chaincode args
*/
func CcQuery(ccId ,functionName string,ccArgs [][]byte) []byte {
	configOpt := getConfig()
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Printf("Failed to create new SDK: %s \n", err)
	}
	defer sdk.Close()
	//prepare channel client context using client context
	clientChannelContext := sdk.ChannelContext(MychannelID, fabsdk.WithUser("Admin"), fabsdk.WithOrg(MyorgName))
	// Channel client is used to query and execute transactions (Org1 is default org)
	client, err := channel.New(clientChannelContext)
	if err != nil {
		fmt.Printf("Failed to create new channel client: %s \n", err)
	}
	return querycc(client,ccId,functionName,ccArgs)
}
/*
Parameters:
  ccId: is chaincode name(such mycc)
  functionName: is invoke chaincode functionName (such query)
  ccArgs: is invoke chaincode args
*/
func CcInvoke(ccId ,functionName string,ccArgs [][]byte) []byte {

	configOpt := getConfig()
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Printf("Failed to create new SDK: %s \n", err)
	}
	defer sdk.Close()
	//prepare channel client context using client context
	clientChannelContext := sdk.ChannelContext(MychannelID, fabsdk.WithUser("Admin"), fabsdk.WithOrg(MyorgName))
	// Channel client is used to query and execute transactions (Org1 is default org)
	client, err := channel.New(clientChannelContext)
	if err != nil {
		fmt.Printf("Failed to create new channel client: %s \n", err)
	}
	return executeCc(client,ccId,functionName,ccArgs)
}


func CcCreate(){
	configOpt := getConfig()
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Println("Failed to create new SDK: ", err)
		return
	}
	defer sdk.Close()


	//prepare context
	adminContext := sdk.Context(fabsdk.WithUser(orgAdmin), fabsdk.WithOrg("Org1"))

	// Org resource management client
	orgResMgmt, err := resmgmt.New(adminContext)
	if err != nil {
		fmt.Println("Failed to create new resource management client: ", err)
		return
	}
	// Create chaincode package for example cc
	response:= createCc(orgResMgmt)
	if response ==nil{
		return
	}
	fmt.Println("createCc txId",response.TransactionID)
}


func QueryBlockByHeight(){
	configOpt := getConfig()
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Printf("Failed to create new SDK: %s \n", err)
	}
	defer sdk.Close()
	//prepare channel client context using client context
	clientChannelContext := sdk.ChannelContext(MychannelID, fabsdk.WithUser("Admin"), fabsdk.WithOrg(MyorgName))
	// Channel client is used to query and execute transactions (Org1 is default org)
	client, err := ledger.New(clientChannelContext)
	if err != nil {
		fmt.Printf("Failed to create new channel client: %s \n", err)
	}
	queryBlockByHeight(client,1)
}


func queryBlockByHeight(client *ledger.Client,blockNumber uint64){
	block,err := client.QueryBlock(blockNumber)
	if err != nil{
		fmt.Println("QueryBlock err",err)
		return
	}
	fmt.Println("QueryBlock result",block)
}
func querycc(client *channel.Client, ccId ,functionName string,ccArgs [][]byte,targetEndpoints ...string) []byte {
	response, err := client.Query(channel.Request{ChaincodeID: ccId, Fcn: functionName, Args: ccArgs},
		channel.WithRetry(retry.DefaultChannelOpts),
		channel.WithTargetEndpoints(targetEndpoints...),
	)
	if err != nil {
		fmt.Printf("Failed to query funds: %s \n", err)
	}
	return response.Payload
}

func executeCc(client *channel.Client,ccId ,functionName string,ccArgs [][]byte) []byte {
	response, err := client.Execute(channel.Request{ChaincodeID: ccId, Fcn: functionName, Args: ccArgs},
		channel.WithRetry(retry.DefaultChannelOpts))
	if err != nil {
		fmt.Printf("Failed to move funds: %s", err)
	}
	return response.Payload
}

func installCc(userName,orgName,chaincodePath ,goPath ,ccId, version string,targetFilter fab.TargetFilter)*resmgmt.InstallCCResponse {
	configOpt := getConfig()
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Println("Failed to create new SDK: ", err)
		return nil
	}
	defer sdk.Close()

	//prepare context
	adminContext := sdk.Context(fabsdk.WithUser(userName), fabsdk.WithOrg(orgName))

	// Org resource management client
	orgResMgmt, err := resmgmt.New(adminContext)
	if err != nil {
		fmt.Println("Failed to create new resource management client: ", err)
		return nil
	}
	//CcAddress := "/opt/gopath/src/github.com/fabric-sdk-sample/fabric-sdk-go/test/fixtures/testdata"
	ccPkg, err := packager.NewCCPackage(chaincodePath, goPath)
	if err != nil {
		fmt.Printf("createCc happen err: %s", err)
		return nil
	}
	// Install example cc to org peers
	installCCReq := resmgmt.InstallCCRequest{Name: ccId, Path: chaincodePath, Version: version, Package: ccPkg}
	_, err = orgResMgmt.InstallCC(installCCReq, resmgmt.WithRetry(retry.DefaultResMgmtOpts),resmgmt.WithTargetFilter(targetFilter))//install 是依赖orgId 的peer
	if err != nil {
		fmt.Printf("createCc happen err: %s", err)
		return nil
	}
	return nil
}
func instantiateCc(userName,orgName,chaincodePath  ,ccID, version string,args [][]byte,ccPolicy *common.SignaturePolicyEnvelope,targetFilter fab.TargetFilter)*resmgmt.InstantiateCCResponse {
	configOpt := getConfig()
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Println("Failed to create new SDK: ", err)
		return nil
	}
	defer sdk.Close()

	//prepare context
	adminContext := sdk.Context(fabsdk.WithUser(userName), fabsdk.WithOrg(orgName))

	// Org resource management client
	orgResMgmt, err := resmgmt.New(adminContext)
	if err != nil {
		fmt.Println("Failed to create new resource management client: ", err)
		return nil
	}
	// Set up chaincode policy
	ccPolicy = cauthdsl.SignedByAnyMember([]string{"Org1MSP"})  //注册chaincode policy
	// Org resource manager will instantiate 'example_cc' on channel
	resp, err := orgResMgmt.InstantiateCC(   //InstantiateCC 实例instantiataCC 却是从channels 获取peer
		channelID,
		resmgmt.InstantiateCCRequest{Name: ccID, Path:chaincodePath, Version: version, Args: args, Policy: ccPolicy},
		resmgmt.WithRetry(retry.DefaultResMgmtOpts),
		resmgmt.WithTargetFilter(targetFilter),
	)
	if err!=nil{
		fmt.Printf("createCc happen err: %s", err)
		return nil
	}
	return &resp
}

func upgradeCc(userName,orgName,chaincodePath  ,ccID, version string,args [][]byte,ccPolicy *common.SignaturePolicyEnvelope,targetFilter fab.TargetFilter)*resmgmt.UpgradeCCResponse{
	configOpt := getConfig()
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Println("Failed to create new SDK: ", err)
		return nil
	}
	defer sdk.Close()

	//prepare context
	adminContext := sdk.Context(fabsdk.WithUser(userName), fabsdk.WithOrg(orgName))

	// Org resource management client
	orgResMgmt, err := resmgmt.New(adminContext)
	if err != nil {
		fmt.Println("Failed to create new resource management client: ", err)
		return nil
	}
	// Set up chaincode policy
	ccPolicy = cauthdsl.SignedByAnyMember([]string{"Org1MSP"})  //注册chaincode policy
	// Org resource manager will instantiate 'example_cc' on channel
	resp, err := orgResMgmt.UpgradeCC(   //InstantiateCC 实例instantiataCC 却是从channels 获取peer
		channelID,
		resmgmt.UpgradeCCRequest{Name: ccID, Path:chaincodePath, Version: version, Args: args, Policy: ccPolicy},
		resmgmt.WithRetry(retry.DefaultResMgmtOpts),
		resmgmt.WithTargetFilter(targetFilter),
	)
	if err!=nil{
		fmt.Printf("createCc happen err: %s", err)
		return nil
	}
	return &resp
}


func createCc( orgResMgmt *resmgmt.Client) *resmgmt.InstantiateCCResponse {
	peerFilter := &PeerFilter{URl:[]string{"39.98.194.244:7251"}}
	CcAddress := "/opt/gopath/src/github.com/fabric-sdk-sample/fabric-sdk-go/test/fixtures/testdata"
	ccPkg, err := packager.NewCCPackage("github.com/example_cc", CcAddress)
	if err != nil {
		fmt.Printf("createCc happen err NewCCPackage: %s", err)
		return nil
	}
	//Install example cc to org peers
	installCCReq := resmgmt.InstallCCRequest{Name: "testcc", Path: "github.com/example_cc", Version: "1.0", Package: ccPkg}
	_, err = orgResMgmt.InstallCC(installCCReq, resmgmt.WithRetry(retry.DefaultResMgmtOpts),)//install 是依赖orgId 的peer
	if err != nil {
		fmt.Printf("createCc happen err InstallCC: %s", err)
		return nil
	}
	return nil

	// Set up chaincode policy
	ccPolicy := cauthdsl.SignedByAnyMember([]string{"Org1MSP"})  //注册chaincode policy
	// Org resource manager will instantiate 'example_cc' on channel
	resp, err := orgResMgmt.InstantiateCC(   //InstantiateCC 实例instantiataCC 却是从channels 获取peer
		channelID,
		resmgmt.InstantiateCCRequest{Name: "testcc", Path: "github.com/chaincode/chaincode_example02/go", Version: "1.0", Args: integration.ExampleCCInitArgs(), Policy: ccPolicy},
		resmgmt.WithRetry(retry.DefaultResMgmtOpts),
		resmgmt.WithTargetFilter(peerFilter),

	)
	if err!=nil{
		fmt.Printf("createCc happen err InstantiateCC: %s", err)
		return nil
	}
	return &resp
}