package fabric_sdk

import (
	"fmt"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/client/channel"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/client/ledger"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/common/errors/retry"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/common/providers/core"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/core/config"
	"github.com/fabric-sdk-sample/fabric-sdk-go/pkg/fabsdk"
	"github.com/fabric-sdk-sample/fabric-sdk-go/test/metadata"
	"path/filepath"
)

var (
	MychannelID          = "mychannel"
	MyorgName            = "org2"
	MyccID               = "mycc"
	MydefaultQueryArgs   = [][]byte{[]byte("b")}
	MydefaultexecuteArgs = [][]byte{[]byte("b"), []byte("a"), []byte("10")}
)
// GetConfigPath returns the path to the named config fixture file
func GetConfigPath(filename string) string {
	const configPath = "config"
	return filepath.Join(metadata.GetProjectPath(), configPath, filename)
}

/*
	这是我们当前读取配置文件的路径，系统运行目录/config/filename
*/
func getConfig(filename string) core.ConfigProvider {
	configPath := GetConfigPath(filename)
	return config.FromFile(configPath)
}

/*
Parameters:
  ccId: is chaincode name(such mycc)
  functionName: is query chaincode functionName (such query)
  ccArgs: is query chaincode args
*/
func CcQuery(ccId ,functionName string,ccArgs [][]byte) (channel.Response,error) {
	configOpt := getConfig("config_e2e.yaml")
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Printf("Failed to create new SDK: %s \n", err)
	}
	defer sdk.Close()
	//prepare channel client context using client context
	clientChannelContext := sdk.ChannelContext(MychannelID, fabsdk.WithUser("Admin"), fabsdk.WithOrg(MyorgName))
	// Channel client is used to query and execute transactions (Org1 is default org)
	client, err := channel.New(clientChannelContext)
	if err != nil {
		fmt.Printf("Failed to create new channel client: %s \n", err)
	}
	return querycc(client,ccId,functionName,ccArgs)
}
/*
Parameters:
  ccId: is chaincode name(such mycc)
  functionName: is invoke chaincode functionName (such query)
  ccArgs: is invoke chaincode args
*/
func CcInvoke(ccId ,functionName string,ccArgs [][]byte) (channel.Response,error) {

	configOpt := getConfig("config_e2e.yaml")
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Printf("Failed to create new SDK: %s \n", err)
	}
	defer sdk.Close()
	//prepare channel client context using client context
	clientChannelContext := sdk.ChannelContext(MychannelID, fabsdk.WithUser("Admin"), fabsdk.WithOrg(MyorgName))
	// Channel client is used to query and execute transactions (Org1 is default org)
	client, err := channel.New(clientChannelContext)
	if err != nil {
		fmt.Printf("Failed to create new channel client: %s \n", err)
	}
	return executeCc(client,ccId,functionName,ccArgs)
}


func QueryBlockByHeight(){
	configOpt := getConfig("config_e2e.yaml")
	sdk, err := fabsdk.New(configOpt)
	if err != nil {
		fmt.Printf("Failed to create new SDK: %s \n", err)
	}
	defer sdk.Close()
	//prepare channel client context using client context
	clientChannelContext := sdk.ChannelContext(MychannelID, fabsdk.WithUser("Admin"), fabsdk.WithOrg(MyorgName))
	// Channel client is used to query and execute transactions (Org1 is default org)
	client, err := ledger.New(clientChannelContext)
	if err != nil {
		fmt.Printf("Failed to create new channel client: %s \n", err)
	}
	queryBlockByHeight(client,1)
}


func queryBlockByHeight(client *ledger.Client,blockNumber uint64){
	block,err := client.QueryBlock(blockNumber)
	if err != nil{
		fmt.Println("QueryBlock err",err)
		return
	}
	fmt.Println("QueryBlock result",block)
}
func querycc(client *channel.Client, ccId ,functionName string,ccArgs [][]byte,targetEndpoints ...string) (channel.Response,error){
	return  client.Query(channel.Request{ChaincodeID: ccId, Fcn: functionName, Args: ccArgs},
		channel.WithRetry(retry.DefaultChannelOpts),
		channel.WithTargetEndpoints(targetEndpoints...),
	)


}

func executeCc(client *channel.Client,ccId ,functionName string,ccArgs [][]byte) (channel.Response,error) {
	return client.Execute(channel.Request{ChaincodeID: ccId, Fcn: functionName, Args: ccArgs},
		channel.WithRetry(retry.DefaultChannelOpts))

}


