package fabric_sdk

import (
	"fmt"
	"testing"
)

/*
 	进行简单的chaincode invoke,query sample
*/

func TestCcQuery(t *testing.T) {
	res,err := CcQuery("mycc","cert",nil)   //共享汇table query
	if err!=nil{
		fmt.Println("CcQuery execute err: ",err)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",res.ChaincodeStatus,string(res.Payload))
	}
	fmt.Println("===============================schema get query==================================================")
	fmt.Println("")
	//resSchema,errSchema := CcQuery("mycc","schema",[][]byte{[]byte("get"),[]byte("alibusi")})   //共享汇table query
	//if err!=nil{
	//	fmt.Println("CcQuery execute err: ",errSchema)
	//}else{
	//	fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	//}
}

var input =  `[{"id":"1","aname":"aaa","atype":"aaa","alinkman":"cc","aader":"aader","amob":"1234567890","backup":{"id":"1","aname":"alibusi","atype":"type","alinkman":"aaa","aader":"aader","amob":"1234567890"}}]`
var inputUp =  `{"id":"1","aname":"bbb","atype":"ccc","backup":{"id":"33","aname":"ertyui","atype":"cvghjui","alinkman":"aaa","amob":"33333333333"}}`
func TestCcInvoke(t *testing.T) {
	fmt.Println("===================================put schema====================================================")
	fmt.Println()
	resSchema,errSchema :=  CcInvoke("mycc","schema",[][]byte{[]byte("put"),[]byte("alibusi"),[]byte(input)})
	if errSchema!=nil{
		fmt.Println("CcQuery execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}


	fmt.Println("===================================put table====================================================")
	fmt.Println()
	resput,errput :=  CcInvoke("mycc","put",[][]byte{[]byte("alibusi"),[]byte(input)})
	if errput!=nil{
		fmt.Println("CcQuery execute err: ",errput)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resput.ChaincodeStatus,string(resput.Payload))
	}

	fmt.Println("===================================get111111111 table====================================================")
	fmt.Println()
	resGet,errGet :=  CcInvoke("mycc","get",[][]byte{[]byte("alibusi"),[]byte(`{"id":"1"}`)})
	if errGet!=nil{
		fmt.Println("CcQuery execute err: ",errGet)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resGet.ChaincodeStatus,string(resGet.Payload))
	}

	fmt.Println("===================================update table====================================================")
	fmt.Println()
	res,err :=  CcInvoke("mycc","update",[][]byte{[]byte("alibusi"),[]byte(inputUp)})
	if err!=nil{
		fmt.Println("CcQuery execute err: ",err)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",res.ChaincodeStatus,string(res.Payload))
	}

	fmt.Println("===================================get22222222222 table====================================================")
	fmt.Println()
	resGet2,errGet2 :=  CcInvoke("mycc","get",[][]byte{[]byte("alibusi"),[]byte(inputUp)})
	if errGet2!=nil{
		fmt.Println("CcQuery execute err: ",errGet2)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resGet2.ChaincodeStatus,string(resGet2.Payload))
	}

}
/*
共享慧chaincode测试说明：
当前共享慧共有两部分内容：
1：schema 部分。
分为三个操作function
(1)put:存储表结构，用于table put(具体表数据存储时) or update(具体表数据结构更新)时进行table 结构校验。每条schema 数据由table name 为key。
args[0]区分是schema 的操作类型，分为put,update,get 三种.args[1],代表table name,唯一值不可重复。args[2]具体的table struct
(2)update:更新表结构
args[0]区分是schema 的操作类型，分为put,update,get 三种.args[1],代表table name,唯一值不可重复。args[2]具体update的table struct
(3)get :获得表结构
args[0]区分是schema 的操作类型，分为put,update,get 三种.args[1],代表table name,唯一值不可重复。
2：table data 部分（数据存储部分，数据存在表结构，所以在进行table data 操作前，需要存在对应的table schema）
(1)put:存储具体的表数据，支持批量存储。
put 操作时，args[0] 表示table name,我们会根据table name,查询对应的schema 数据，用于对table data 进行格式校验。所以我们在进行put 操作前，一定要提前进行对应的
schema table struct put operation.args[1],表示具体的存储表数据，args[1]可以是批量表数据，也可以是单条表数据。每条数据除了必须满足对应的表结构，必须存在id 字段
id 字段是一条数据的唯一key.不能重复。
(2)update :修改表数据，不支持批量数据
update 操作时，args[0] 表示table name,args[1]表示要修改的数据，同样必须存在id 字段
(3)get :根据表id字段查询表数据
get操作时，args[0] 表示table name,args[1],表示要查询数据的id
*/


/*
	一个新表格数据完整的测试顺序
1：Schema,put
2:Data put
3:Data get
*/


/*
我们常遇到的错误：
1：重复执行同一张表的schema put 操作。
解决方法修改args[1] 的table name
error message:the parameter ** already exists and cannot be added
2:重复执行同一张表的Data put 操作。
解决方法，更新args[1]的存储数据的id 字段值
error message: ** data already exist
3：直接执行表的Data put 操作，而没有执行对应表结构的Schema put操作。
error message: GetSchema data fail,please check your parameters
解决方法，先执行对应表结构的Schema put操作
*/
func TestSchemaPut(t *testing.T){
	resSchema,errSchema :=  CcInvoke("mycc","schema",[][]byte{[]byte("put"),[]byte("alibusi2"),[]byte(input)})
	if errSchema!=nil{
		fmt.Println("CcInvoke execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}

func TestSchemaGet(t *testing.T){
	resSchema,errSchema :=  CcQuery("mycc","schema",[][]byte{[]byte("get"),[]byte("alibusi2")})
	if errSchema!=nil{
		fmt.Println("CcQuery execute err: ",errSchema)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}

/*
	update Schema 只对新数据格式是否符合map[string]interface{} 的格式，以及更新的数据是否存在
*/
func TestSchemaUpdate(t *testing.T){
	resSchema,errSchema :=  CcInvoke("mycc","schema",[][]byte{[]byte("update"),[]byte("alibusi2"),[]byte(input)})
	if errSchema!=nil{
		fmt.Println("CcInvoke execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}
/*
   ----------------------------------------------------------Data Operation------------------------------------------------------------
*/
func TestDataPut(t *testing.T){
	resSchema,errSchema :=  CcInvoke("mycc","put",[][]byte{[]byte("alibusi2"),[]byte(input)})
	if errSchema!=nil{
		fmt.Println("CcInvoke execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}

func TestDataGet(t *testing.T){
	resSchema,errSchema :=  CcQuery("mycc","get",[][]byte{[]byte("alibusi2"),[]byte(`{"id":"1"}`)})
	if errSchema!=nil{
		fmt.Println("CcQuery execute err: ",errSchema)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}

func TestDataUpdate(t *testing.T){
	resSchema,errSchema :=  CcInvoke("mycc","update",[][]byte{[]byte("alibusi2"),[]byte(inputUp)})
	if errSchema!=nil{
		fmt.Println("CcInvoke execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}