package fabric_sdk

import (
	"fmt"
	"testing"
)

/*
	本测试文件支持共享惠增加权限部分的测试
*/


var (
	authInput = `{"fields":{"alinkman":{"val":"Alinkman","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"amob":{"val":"Amob","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":
{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},"aname":{"val":"Aname","auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}}}},"atype":{"val":"Atype","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"backup":{"val":{"aname":"Aname","atype":"Atype"},"auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"id":{"val":"1","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}}},"schema_auth":{"read":{"users":null,"roles":null},
"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}},"auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":0}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":0}}}}
`

	inputUpdate = `{"id":"1","aname":"alibusi","atype":"newtype","alinkman":"newalinkman",
"amob":"new1234567890","backup":{"aname":"alibusi","atype":"type"}}`

	inputarrary = `[{"id":"1","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}},{"id":"2","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}}]`
	ccId = "mycc"
)


func TestSchemaPutAuth(t *testing.T){
	resSchema,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("put"),[]byte("alibusii"),[]byte(authInput)})
	if errSchema!=nil{
		fmt.Println("CcInvoke execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}

func TestSchemaGetAuth(t *testing.T){
	resSchema,errSchema :=  CcQuery(ccId,"schema",[][]byte{[]byte("get"),[]byte("alibusii")})
	if errSchema!=nil{
		fmt.Println("CcQuery execute err: ",errSchema)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}

/*
	update Schema 只对新数据格式是否符合map[string]interface{} 的格式，以及更新的数据是否存在
*/
func TestSchemaUpdateAuth(t *testing.T){
	resSchema,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("update"),[]byte("alibusii"),[]byte(authInput)})
	if errSchema!=nil{
		fmt.Println("CcInvoke execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}
/*
   ----------------------------------------------------------Data Operation------------------------------------------------------------
*/
func TestDataPutAuth(t *testing.T){
	resSchema,errSchema :=  CcInvoke(ccId,"put",[][]byte{[]byte("alibusii"),[]byte(inputarrary)})
	if errSchema!=nil{
		fmt.Println("CcInvoke execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}

func TestDataGetAuth(t *testing.T){
	resSchema,errSchema :=  CcQuery(ccId,"get",[][]byte{[]byte("alibusii"),[]byte(`{"id":"1"}`)})
	if errSchema!=nil{
		fmt.Println("CcQuery execute err: ",errSchema)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}

func TestDataUpdateAuth(t *testing.T){
	resSchema,errSchema :=  CcInvoke(ccId,"update",[][]byte{[]byte("alibusii"),[]byte(inputUpdate)})
	if errSchema!=nil{
		fmt.Println("CcInvoke execute err: ",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
}



/*
	当数据全部正确时的测试数据，当前用户为Admin@org2.example.com
*/
func TestAuthRight(t *testing.T){
	taleName := "alibusiRight" //我们在每次测试的时候需要保证我们的table name 不重复
	var (
		authInput = `{"fields":{"alinkman":{"val":"Alinkman","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"amob":{"val":"Amob","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":
{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},"aname":{"val":"Aname","auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}}}},"atype":{"val":"Atype","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"backup":{"val":{"aname":"Aname","atype":"Atype"},"auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"id":{"val":"1","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}}},"schema_auth":{"read":{"users":null,"roles":null},
"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}},"auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":0}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":0}}}}
`

		inputUpdate = `{"id":"1","aname":"alibusi","atype":"newtype","alinkman":"newalinkman",
"amob":"new1234567890","backup":{"aname":"alibusi","atype":"type"}}`

		inputarrary = `[{"id":"1","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}},{"id":"2","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}}]`
	)

	fmt.Println("=============================================put schema======================================================")
	resSchema,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("put"),[]byte(taleName),[]byte(authInput)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================get schema======================================================")
	fmt.Println()
	resSchemaget,errSchemaget :=  CcQuery(ccId,"schema",[][]byte{[]byte("get"),[]byte(taleName)})
	if errSchemaget!=nil{
		t.Errorf("CcQuery execute err : %s",errSchemaget)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchemaget.ChaincodeStatus,string(resSchemaget.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================update schema======================================================")
	fmt.Println()
	resSchemaup,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("update"),[]byte(taleName),[]byte(authInput)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchemaup.ChaincodeStatus,string(resSchemaup.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================put data======================================================")
	fmt.Println()
	resData,errData :=  CcInvoke(ccId,"put",[][]byte{[]byte(taleName),[]byte(inputarrary)})
	if errData!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resData.ChaincodeStatus,string(resData.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================get data======================================================")
	fmt.Println()
	resDataget,errSchema :=  CcQuery(ccId,"get",[][]byte{[]byte(taleName),[]byte(`{"id":"1"}`)})
	if errSchema!=nil{
		t.Errorf("CcQuery execute err : %s",errSchema)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resDataget.ChaincodeStatus,string(resDataget.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================update data======================================================")
	fmt.Println()
	resDataup,errSchema :=  CcInvoke(ccId,"update",[][]byte{[]byte(taleName),[]byte(inputUpdate)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resDataup.ChaincodeStatus,string(resDataup.Payload))
	}
}

/*
	主要测试schema Update时的权限测试，当前用户为Admin@org2.example.com
*/
func TestAuthSchemaWrite(t *testing.T){
	taleName := "alibusiSUp" //我们在每次测试的时候需要保证我们的table name 不重复
	var (
		authInput = `{"fields":{"alinkman":{"val":"Alinkman","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"amob":{"val":"Amob","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":
{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},"aname":{"val":"Aname","auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}}}},"atype":{"val":"Atype","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"backup":{"val":{"aname":"Aname","atype":"Atype"},"auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"id":{"val":"1","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}}},"schema_auth":{"read":{"users":null,"roles":null},
"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}},"auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":0}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":0}}}}
`

	)

	fmt.Println("=============================================put schema======================================================")
	resSchema,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("put"),[]byte(taleName),[]byte(authInput)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================get schema======================================================")
	fmt.Println()
	resSchemaget,errSchemaget :=  CcQuery(ccId,"schema",[][]byte{[]byte("get"),[]byte(taleName)})
	if errSchemaget!=nil{
		t.Errorf("CcQuery execute err : %s",errSchemaget)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchemaget.ChaincodeStatus,string(resSchemaget.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================update schema======================================================")
	fmt.Println()
	resSchemaup,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("update"),[]byte(taleName),[]byte(authInput)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchemaup.ChaincodeStatus,string(resSchemaup.Payload))
	}
}

/*
	主要测试Data put时的权限测试，当前用户为Admin@org2.example.com
*/
func TestAuthDataPut(t *testing.T){
	taleName := "alibusiDataPut" //我们在每次测试的时候需要保证我们的table name 不重复
	var (
		authInput = `{"fields":{"alinkman":{"val":"Alinkman","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"amob":{"val":"Amob","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":
{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},"aname":{"val":"Aname","auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}}}},"atype":{"val":"Atype","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"backup":{"val":{"aname":"Aname","atype":"Atype"},"auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"id":{"val":"1","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}}},"schema_auth":{"read":{"users":null,"roles":null},
"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}},"auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":0}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"User1@org2.example.com":0}}}}
`
		inputarrary = `[{"id":"1","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}},{"id":"2","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}}]`
	)

	fmt.Println("=============================================put schema======================================================")
	resSchema,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("put"),[]byte(taleName),[]byte(authInput)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================get schema======================================================")
	fmt.Println()
	resSchemaget,errSchemaget :=  CcQuery(ccId,"schema",[][]byte{[]byte("get"),[]byte(taleName)})
	if errSchemaget!=nil{
		t.Errorf("CcQuery execute err : %s",errSchemaget)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchemaget.ChaincodeStatus,string(resSchemaget.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================put data======================================================")
	fmt.Println()
	resData,errData :=  CcInvoke(ccId,"put",[][]byte{[]byte(taleName),[]byte(inputarrary)})
	if errData!=nil{
		t.Errorf("CcInvoke execute err : %s",errData)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resData.ChaincodeStatus,string(resData.Payload))
	}
}

/*
主要测试Data Update时的权限测试,主要测试字段的权限控制，当前用户为Admin@org2.example.com
*/
func TestAuthDataUpdate(t *testing.T){
	taleName := "alibusiDataUp1" //我们在每次测试的时候需要保证我们的table name 不重复
	var (
		authInput = `{"fields":{"alinkman":{"val":"Alinkman","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"amob":{"val":"Amob","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":
{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},"aname":{"val":"Aname","auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}}}},"atype":{"val":"Atype","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"backup":{"val":{"aname":"Aname","atype":"Atype"},"auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"id":{"val":"1","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}}},"schema_auth":{"read":{"users":null,"roles":null},
"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}},"auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":0}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":0}}}}
`
		authInputUp = `{"Parameters":{"alinkman":{"Value":"Alinkman","Read":["user"],"Write":["admin"]},"amob":{"Value":"Amob","Read":[],"Write":[]},"aname":
{"Value":"Aname","Read":["admin"],"Write":["admin"]},"atype":{"Value":"Atype","Read":["admin"],"Write":["admin"]},"backup":{"Value":{"aname":
{"Value":"Aname","Read":["admin"],"Write":["admin"]},"atype":{"Value":"Atype","Read":["admin"],"Write":["admin"]}},"Read":["admin"],"Write":["admin"]},
"id":{"Value":"1","Read":[],"Write":[]}},"Read":["Admin@org2.example.com"],"Write":["User1@org2.example.com"],"SchemaWrite":["Admin@org2.example.com"]}`

		inputUpdate = `{"id":"1","aname":"alibusi","atype":"newtype","alinkman":"newalinkman",
"amob":"new1234567890","backup":{"aname":"alibusi","atype":"type"}}`

		inputarrary = `[{"id":"1","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}},{"id":"2","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}}]`
	)

	fmt.Println("=============================================put schema======================================================")
	resSchema,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("put"),[]byte(taleName),[]byte(authInput)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================get schema======================================================")
	fmt.Println()
	resSchemaget,errSchemaget :=  CcQuery(ccId,"schema",[][]byte{[]byte("get"),[]byte(taleName)})
	if errSchemaget!=nil{
		t.Errorf("CcQuery execute err : %s",errSchemaget)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchemaget.ChaincodeStatus,string(resSchemaget.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================put data======================================================")
	fmt.Println()
	resData,errData :=  CcInvoke(ccId,"put",[][]byte{[]byte(taleName),[]byte(inputarrary)})
	if errData!=nil{
		t.Errorf("CcInvoke execute err : %s",errData)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resData.ChaincodeStatus,string(resData.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================update schema======================================================")
	fmt.Println()
	resSchemaup,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("update"),[]byte(taleName),[]byte(authInputUp)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchemaup.ChaincodeStatus,string(resSchemaup.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================update data======================================================")
	fmt.Println()
	resDataUp,errData :=  CcInvoke(ccId,"update",[][]byte{[]byte(taleName),[]byte(inputUpdate)})
	if errData!=nil{
		t.Errorf("CcInvoke execute err : %s",errData)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resDataUp.ChaincodeStatus,string(resDataUp.Payload))
	}
}

/*
	主要测试Data get时的权限测试,主要测试字段的权限控制，当前用户为Admin@org2.example.com
*/
func TestAuthDataGet(t *testing.T){
	taleName := "alibusiDataGet" //我们在每次测试的时候需要保证我们的table name 不重复
	var (
		authInput = `{"fields":{"alinkman":{"val":"Alinkman","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"amob":{"val":"Amob","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":
{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},"aname":{"val":"Aname","auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}}}},"atype":{"val":"Atype","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"backup":{"val":{"aname":"Aname","atype":"Atype"},"auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}},
"id":{"val":"1","auth":{"read":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}}}},"schema_auth":{"read":{"users":null,"roles":null},
"write":{"users":{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}}},"auth":{"read":{"users":
{"Admin@org2.example.com":0},"roles":{"Admin@org2.example.com":1582988515}},"write":{"users":{"Admin@org2.example.com":0},"roles":
{"Admin@org2.example.com":0}}}}
`
		inputget = `{"id":"1"}`

		inputarrary = `[{"id":"1","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}},{"id":"2","aname":"alibusi","atype":"type","alinkman":"alinkman",
"amob":"1234567890","backup":{"aname":"alibusi","atype":"type"}}]`
	)

	fmt.Println("=============================================put schema======================================================")
	resSchema,errSchema :=  CcInvoke(ccId,"schema",[][]byte{[]byte("put"),[]byte(taleName),[]byte(authInput)})
	if errSchema!=nil{
		t.Errorf("CcInvoke execute err : %s",errSchema)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resSchema.ChaincodeStatus,string(resSchema.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================get schema======================================================")
	fmt.Println()
	resSchemaget,errSchemaget :=  CcQuery(ccId,"schema",[][]byte{[]byte("get"),[]byte(taleName)})
	if errSchemaget!=nil{
		t.Errorf("CcQuery execute err : %s",errSchemaget)
	}else{
		fmt.Printf("Query status %d and payload %s \n",resSchemaget.ChaincodeStatus,string(resSchemaget.Payload))
	}
	fmt.Println()
	fmt.Println("=============================================put data======================================================")
	fmt.Println()
	resData,errData :=  CcInvoke(ccId,"put",[][]byte{[]byte(taleName),[]byte(inputarrary)})
	if errData!=nil{
		t.Errorf("CcInvoke execute err : %s",errData)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resData.ChaincodeStatus,string(resData.Payload))
	}
	fmt.Println()

	fmt.Println()
	fmt.Println("=============================================get data======================================================")
	fmt.Println()
	resDataget,errData :=  CcInvoke(ccId,"get",[][]byte{[]byte(taleName),[]byte(inputget)})
	if errData!=nil{
		t.Errorf("CcInvoke execute err : %s",errData)
	}else{
		fmt.Printf("Invoke status %d and payload %s \n",resDataget.ChaincodeStatus,string(resDataget.Payload))
	}
}
